#!/usr/bin/python3
# 
# Copyright (c) Andrea Micheloni 2017
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import requests
import unittest
from .context import corporatesnorlax
from corporatesnorlax import helpers
import os

logger = logging.getLogger(__name__)

class MockService(object):
    def __init__(self, location, base64):
        self._url = location.get_url()
        
        self._expected_headers = {'accept': 'application/json', 'Authorization': 'Basic ' + str(base64)}
        
        logger.info("Mock initialised with location %s", location)
        
    def _register_uri(self, mock, url, method, status_code=200, json=None, complete_query=False):
        mock.register_uri(url=self._url + url, method=method, complete_qs = complete_query,
                            request_headers=self._expected_headers, 
                            headers= {'Content-Type': 'application/json'}, 
                            json=json, status_code = status_code)
                            
    def _register_POST_uri(self, mock, url, request_json, status_code=200, json=None):
        return self._register_uri_with_method(mock, 'POST', url, request_json, status_code, json)
    
    def _register_PUT_uri(self, mock, url, request_json, status_code=200, json=None):
        return self._register_uri_with_method(mock, 'PUT', url, request_json, status_code, json)
    
    def _register_uri_with_method(self, mock, method, url, request_json, status_code, json):
        complete_url=self._url + url
    
        def _method_matcher(request):
            response = None

            if request.method == str(method) and complete_url[-len(request.path_url):] == request.path_url:
                headers_match = True
            
                for key in self._expected_headers:
                    if key not in request.headers or self._expected_headers[key] != request.headers[key]:
                        headers_match = False
                        logger.error('%s headers to %s not matched: %s', method, url, key)
                
                if headers_match:
                    logger.info('%s to %s, got JSON %s', method, url, request.json())
                    fully_matched = True
                    found_request_json = request.json()
                    
                    if type(request_json).__name__ == 'dict':
                        for key in request_json:
                            if key not in found_request_json or request_json[key] != found_request_json[key]:
                                fully_matched = False
                    else:
                        fully_matched = (request_json == found_request_json)
                
                    if fully_matched:
                        response = requests.Response()
                        def return_json():
                            return json
                        response.json = return_json
                        response.status_code = status_code
            
            return response
                
        mock.add_matcher(_method_matcher)
        
class BaseTestMock(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        unittest.TestCase.__init__(self, *args, **kwargs)
        
        helpers.ConfigurationFiles().set_base_directory(os.path.dirname(os.path.realpath(__file__)) + '/config/')
        
        logger.debug('Initialized configuration_files with base directory %s', helpers.ConfigurationFiles().get_base_directory())
        
        self._all_credentials = helpers.CredentialsHolder()
        
