#!/usr/bin/python3
# 
# Copyright (c) Andrea Micheloni 2017
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import requests
from . import mock

logger = logging.getLogger(__name__)

class MockConfluence(mock.MockService):
    def __init__(self, location):
        mock.MockService.__init__(self, location, 'dXNlci5pZDpoYWxmdGhlYW5zd2Vy')
        
    def register_uris(self, mock):
        logger.info('Registering the URIs to the mock %s', mock)
        
        self._register_uri(mock, '/rest/api/space/unauthorized?expand=description.plain', 'GET', status_code=401)
        self._register_uri(mock, '/rest/api/space/forbidden?expand=description.plain', 'GET', status_code=403)
        self._register_uri(mock, '/rest/api/space/servererror?expand=description.plain', 'GET', status_code=330)

        self._register_uri(mock, '/rest/api/space/existingspace?expand=description.plain', 'GET', json=MockConfluenceUtil().get_space_json(self._url, 'existingspace', 'Space'), complete_query = True)
        self._register_uri(mock, '/rest/api/space/invalidspace?expand=description.plain', 'GET', status_code=404)

        self._register_POST_uri(mock, '/rpc/json-rpc/confluenceservice-v2/getSpacePermissionSets', request_json=['existingspace'], json=MockConfluenceUtil().get_space_permissions_json(self._url))
        self._register_POST_uri(mock, '/rpc/json-rpc/confluenceservice-v2/getSpacePermissionSets', request_json=['invalidspace'], json=MockConfluenceUtil().get_space_permissions_json_error())

        self._register_POST_uri(mock, '/rest/api/space', request_json={'key': 'NEW', 'name': 'New space'}, json=MockConfluenceUtil().get_space_json(self._url, 'NEW', 'New space'))
        self._register_POST_uri(mock, '/rest/api/space', request_json={'key': 'EXISTING', 'name': 'Existing space'}, status_code=400)

        self._register_POST_uri(mock, '/rpc/json-rpc/confluenceservice-v2/addPermissionsToSpace', request_json=[['NAME1', 'NAME2'], 'target', 'SPACE'])
        self._register_POST_uri(mock, '/rpc/json-rpc/confluenceservice-v2/addPermissionsToSpace', request_json=[['NAME1', 'NAME2'], 'target', 'NONEX'], json=MockConfluenceUtil().get_space_permissions_json_error())
        self._register_POST_uri(mock, '/rpc/json-rpc/confluenceservice-v2/addPermissionsToSpace', request_json=[['NAME1', 'NAME2'], 'nonexisting', 'SPACE'], json=MockConfluenceUtil().get_space_permissions_json_error())

        self._register_uri(mock, '/rest/api/content?spaceKey=existingspace&title=Page Title&expand=body.view,version,space', 'GET', json=MockConfluenceUtil().get_content_json(self._url, 'existingspace', 'Page Title'), complete_query = True)

class MockConfluenceUtil(object):
    def get_space_json(self, confluence_url, key, name):
        return {'description': {'_expandable': {'view': ''}, 'plain': {'representation': 'plain', 'value': 'Test Description'}},
                '_links': {'collection': '/rest/api/space', 'self': 'https://www.pulsewiki.eu/confluence/rest/api/space/' + key, 
                           'context': '/confluence', 'base': 'https://www.pulsewiki.eu/confluence', 'webui': '/display/' + key}, 
                '_expandable': {'icon': '', 'metadata': '', 'homepage': '/rest/api/content/019293'}, 
                'key': key, 'type': 'global', 'name': name, 'id': '019293'}

    def get_space_permissions_json(self, confluence_url):
        return [
  {
    'type': 'SETPAGEPERMISSIONS',
    'spacePermissions': [
      {
        'type': 'SETPAGEPERMISSIONS',
        'userName': 'user.name',
        'groupName': None
      },
      {
        'type': 'SETPAGEPERMISSIONS',
        'userName': None,
        'groupName': 'group.orga'
      }
    ]
  },
  {
    'type': 'REMOVEPAGE',
    'spacePermissions': [
      {
        'type': 'REMOVEPAGE',
        'userName': None,
        'groupName': 'group.orga'
      },
      {
        'type': 'REMOVEPAGE',
        'userName': 'user.name',
        'groupName': None
      }
    ]
  },
  {
    'type': 'EDITBLOG',
    'spacePermissions': [
      {
        'type': 'EDITBLOG',
        'userName': 'user.name',
        'groupName': None
      },
      {
        'type': 'EDITBLOG',
        'userName': None,
        'groupName': 'group.orga'
      }
    ]
  },
  {
    'type': 'REMOVEOWNCONTENT',
    'spacePermissions': [
      {
        'type': 'REMOVEOWNCONTENT',
        'userName': 'user.name',
        'groupName': None
      }
    ]
  },
  {
    'type': 'EDITSPACE',
    'spacePermissions': [
      {
        'type': 'EDITSPACE',
        'userName': None,
        'groupName': 'group.orga'
      },
      {
        'type': 'EDITSPACE',
        'userName': None,
        'groupName': 'group.help'
      },
      {
        'type': 'EDITSPACE',
        'userName': 'user.name',
        'groupName': None
      }
    ]
  },
  {
    'type': 'SETSPACEPERMISSIONS',
    'spacePermissions': [
      {
        'type': 'SETSPACEPERMISSIONS',
        'userName': 'user.name',
        'groupName': None
      }
    ]
  },
  {
    'type': 'REMOVEMAIL',
    'spacePermissions': [
      {
        'type': 'REMOVEMAIL',
        'userName': 'user.name',
        'groupName': None
      }
    ]
  },
  {
    'type': 'VIEWSPACE',
    'spacePermissions': [
      {
        'type': 'VIEWSPACE',
        'userName': 'user.name',
        'groupName': None
      },
      {
        'type': 'VIEWSPACE',
        'userName': None,
        'groupName': 'jira-software-users'
      },
      {
        'type': 'VIEWSPACE',
        'userName': None,
        'groupName': 'group.orga'
      },
      {
        'type': 'VIEWSPACE',
        'userName': None,
        'groupName': 'confluence-users'
      },
      {
        'type': 'VIEWSPACE',
        'userName': None,
        'groupName': 'group.help'
      }
    ]
  },
  {
    'type': 'REMOVEBLOG',
    'spacePermissions': [
      {
        'type': 'REMOVEBLOG',
        'userName': 'user.name',
        'groupName': None
      },
      {
        'type': 'REMOVEBLOG',
        'userName': None,
        'groupName': 'group.orga'
      }
    ]
  },
  {
    'type': 'EXPORTPAGE',
    'spacePermissions': [
      {
        'type': 'EXPORTPAGE',
        'userName': 'user.name',
        'groupName': None
      }
    ]
  },
  {
    'type': 'COMMENT',
    'spacePermissions': [
      {
        'type': 'COMMENT',
        'userName': None,
        'groupName': 'group.orga'
      },
      {
        'type': 'COMMENT',
        'userName': None,
        'groupName': 'confluence-users'
      },
      {
        'type': 'COMMENT',
        'userName': None,
        'groupName': 'group.help'
      },
      {
        'type': 'COMMENT',
        'userName': 'user.name',
        'groupName': None
      },
      {
        'type': 'COMMENT',
        'userName': None,
        'groupName': 'jira-software-users'
      }
    ]
  },
  {
    'type': 'CREATEATTACHMENT',
    'spacePermissions': [
      {
        'type': 'CREATEATTACHMENT',
        'userName': 'user.name',
        'groupName': None
      },
      {
        'type': 'CREATEATTACHMENT',
        'userName': None,
        'groupName': 'group.orga'
      },
      {
        'type': 'CREATEATTACHMENT',
        'userName': None,
        'groupName': 'group.help'
      }
    ]
  },
  {
    'type': 'REMOVEATTACHMENT',
    'spacePermissions': [
      {
        'type': 'REMOVEATTACHMENT',
        'userName': 'user.name',
        'groupName': None
      },
      {
        'type': 'REMOVEATTACHMENT',
        'userName': None,
        'groupName': 'group.orga'
      }
    ]
  },
  {
    'type': 'REMOVECOMMENT',
    'spacePermissions': [
      {
        'type': 'REMOVECOMMENT',
        'userName': None,
        'groupName': 'group.orga'
      },
      {
        'type': 'REMOVECOMMENT',
        'userName': 'user.name',
        'groupName': None
      }
    ]
  },
  {
    'type': 'EXPORTSPACE',
    'spacePermissions': [
      {
        'type': 'EXPORTSPACE',
        'userName': 'user.name',
        'groupName': None
      }
    ]
  }
]

    def get_space_permissions_json_error(self):
        return {'error': {'code': 500, 'data': "LONGMESSAGE", 
                'message': "...space with key 'test' not found"}, 'id': None, 'jsonrpc': '2.0'}
        
    def get_content_json(self, confluence_url, space_key, title):
        return {'results': [{'extensions': {'position': 'none'}, 'body': {'view': {'_expandable': {'content': '/rest/api/content/000000', 'webresource': ''}, 'representation': 'storage', 'value': '<h1>Content</h1>'}, '_expandable': {'styled_view': '', 'storage': '', 'anonymous_export_view': '', 'editor': '', 'export_view': ''}}, '_links': {'tinyui': '/x/ssdf', 'edit': '/pages/resumedraft.action?draftId=10101', 'webui': '/display/'+space_key+'/'+title, 'self': confluence_url + '/rest/api/content/3966245'}, 'type': 'page', 'title': title, 'space': {'_links': {'webui': '/display/' + space_key, 'self': confluence_url + '/rest/api/space/' + space_key}, '_expandable': {'metadata': '', 'homepage': '/rest/api/content/23984', 'description': '', 'icon': ''}, 'type': 'global', 'id': 298364983, 'key': space_key, 'name': 'Space_name ' + space_key}, 'id': '236498', '_expandable': {'metadata': '', 'operations': '', 'container': '/rest/api/space/' + space_key, 'ancestors': '', 'children': '/rest/api/content/2342344/child', 'history': '/rest/api/content/234234234/history', 'descendants': '/rest/api/content/435345/descendant'}, 'status': 'current', 'version': {'when': '2017-09-12T22:06:24.556+02:00', '_links': {'self': confluence_url+'/rest/experimental/content/234235235/version/7'}, 'minorEdit': False, 'number': 7, 'by': {'_links': {'self': confluence_url + '/rest/experimental/user?key=2983606325320097325'}, 'profilePicture': {'path': '/confluence/download/attachments/928635625/user-avatar', 'isDefault': False, 'width': 48, 'height': 48}, 'type': 'known', 'username': 'user.name', 'displayName': 'User Name', 'userKey': '983750293709230957'}, '_expandable': {'content': '/rest/api/content/345435'}, 'message': '', 'hidden': False}}], 'size': 1, '_links': {'context': '/confluence', 'base': confluence_url, 'self': confluence_url + '/rest/api/content?spaceKey='+space_key+'&expand=body.view,version,space&title=' + title}, 'start': 0, 'limit': 25}
        
