#!/usr/bin/python3
# 
# Copyright (c) Andrea Micheloni 2017
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import requests
from . import mock

logger = logging.getLogger(__name__)

# https://docs.atlassian.com/atlassian-crowd/latest/REST/
class MockCrowd(mock.MockService):
    def __init__(self, location):
        mock.MockService.__init__(self, location, 'Y2hvc2VuLXVzZXJuYW1lOmh1bnRlcjI=')
    
    def register_uris(self, mock):
        logger.info('Registering the URIs to the mock %s', mock)
        
        self._register_uri(mock, '/invalid-response/', 'GET', status_code=412)
        self._register_POST_uri(mock, '/invalid-response/', request_json = {}, status_code = 412)
        self._register_uri(mock, '/rest/usermanagement/1/user?username=unauthorized&expand=attributes', 'GET', status_code=401)
        self._register_uri(mock, '/rest/usermanagement/1/user?username=forbidden&expand=attributes', 'GET', status_code=403)
        self._register_uri(mock, '/rest/usermanagement/1/user?username=servererror&expand=attributes', 'GET', status_code=330)
        self._register_uri(mock, '/rest/usermanagement/1/user?username=existing.user&expand=attributes', 'GET', json=MockUserCrowdUtil().get_user_json(self._url))
        self._register_uri(mock, '/rest/usermanagement/1/user?username=invalid.user&expand=attributes', 'GET', status_code=404)
        self._register_POST_uri(mock, '/rest/usermanagement/1/user?expand=attributes', request_json={'name': 'new.user','password': {'value': 'pass!'},'active': True,'first-name': 'New','last-name': 'User','display-name': 'New User','email': 'sample@user.cool'}, json=MockUserCrowdUtil().get_user_json(self._url, name='new.user'), status_code = 201)
        self._register_POST_uri(mock, '/rest/usermanagement/1/user?expand=attributes', request_json={'name': 'existing.user', 'active': True}, status_code = 400)
        self._register_POST_uri(mock, '/rest/usermanagement/1/user/attribute?username=existing.user', request_json={'attributes':[{'name': 'otheratt', 'values': ['otherValue']}]}, status_code = 204)
        self._register_POST_uri(mock, '/rest/usermanagement/1/user/attribute?username=invalid.user', request_json={'attributes':[{'name': 'attribute', 'values': ['value']}]}, status_code = 404)
        self._register_POST_uri(mock, '/rest/usermanagement/1/user/group/direct?username=existing.user', request_json={'name':'existing.group'}, status_code = 201)
        self._register_POST_uri(mock, '/rest/usermanagement/1/user/group/direct?username=invalid.user', request_json={'name':'existing.group'}, status_code = 404)
        self._register_POST_uri(mock, '/rest/usermanagement/1/user/group/direct?username=existing.user', request_json={'name':'invalid.group'}, status_code = 400)
        self._register_POST_uri(mock, '/rest/usermanagement/1/user/group/direct?username=connected.user', request_json={'name':'existing.group'}, status_code = 409)
        self._register_POST_uri(mock, '/rest/usermanagement/1/user/mail/usernames?email=sample%40user.cool', request_json={}, status_code = 204)
        self._register_POST_uri(mock, '/rest/usermanagement/1/user/mail/usernames?email=invalid%40mail.tld', request_json={}, status_code = 404)
        self._register_uri(mock, '/rest/usermanagement/1/group?groupname=existing.group&expand=attributes', 'GET', json=MockGroupCrowdUtil().get_group_json(self._url, True))
        self._register_uri(mock, '/rest/usermanagement/1/group?groupname=existing.group', 'GET', json=MockGroupCrowdUtil().get_group_json(self._url, False), complete_query = True)
        self._register_uri(mock, '/rest/usermanagement/1/group?groupname=invalid.group', 'GET', status_code=404)
        self._register_uri(mock, '/rest/usermanagement/1/group/child-group/direct?groupname=existing.group', 'GET', json=MockGroupCrowdUtil().get_child_groups_of_group_json(self._url))
        self._register_uri(mock, '/rest/usermanagement/1/group/child-group/direct?groupname=invalid.group', 'GET', status_code=404)
        self._register_uri(mock, '/rest/usermanagement/1/group/parent-group/direct?groupname=existing.group', 'GET', json=MockGroupCrowdUtil().get_parent_groups_of_group_json(self._url))
        self._register_uri(mock, '/rest/usermanagement/1/group/parent-group/direct?groupname=invalid.group', 'GET', status_code=404)
        self._register_uri(mock, '/rest/usermanagement/1/group/user/nested?groupname=existing.group', 'GET', json=MockGroupCrowdUtil().get_nested_members_of_group_json(self._url))
        self._register_uri(mock, '/rest/usermanagement/1/group/user/nested?groupname=invalid.group', 'GET', status_code=404)
        self._register_POST_uri(mock, '/rest/usermanagement/1/group', request_json={'name': 'new.group', 'type': 'GROUP', 'active': True}, json=MockGroupCrowdUtil().get_group_json(self._url, False, name='new.group'), status_code = 201)
        self._register_POST_uri(mock, '/rest/usermanagement/1/group', request_json={'name': 'existing.group', 'type': 'GROUP', 'active': True}, status_code=400)
        self._register_POST_uri(mock, '/rest/usermanagement/1/group/child-group/direct?groupname=parent.group', request_json={'name': 'child.group'}, json=MockGroupCrowdUtil().get_group_json(self._url, False, name='parent.group'), status_code = 201)
        self._register_POST_uri(mock, '/rest/usermanagement/1/group/child-group/direct?groupname=parent.group', request_json={'name': 'invalid.group'}, status_code=400)        
        self._register_POST_uri(mock, '/rest/usermanagement/1/group/child-group/direct?groupname=invalid.group', request_json={'name': 'child.group'}, status_code=404)        

class MockUserCrowdUtil(object):
    def get_user_json(self, crowd_url, name='existing.user'):
        attributes = [{
                "link": {
                    "href": crowd_url + "/rest/usermanagement/1/user/attributes?username=existing.user&attributename=invalidPasswordAttempts",
                    "rel": "self"
                },
                "name": "invalidPasswordAttempts",
                "values": [
                    "0"
                ]
            },
            {
                "link": {
                    "href": crowd_url + "/rest/usermanagement/1/user/attributes?username=existing.user&attributename=invalidPasswordAttempts&attributename=requiresPasswordChange",
                    "rel": "self"
                },
                "name": "requiresPasswordChange",
                "values": [
                    "false"
                ]
            }]
            
        return {
    "expand": "attributes",
    "link": {
        "href": crowd_url + "/rest/usermanagement/1/user?username=existing.user",
        "rel": "self"
    },
    "name": name,
    "password": {
        "link": {
            "href": crowd_url + "/rest/usermanagement/1/user/password?username=existing.user",
            "rel": "edit"
        }
    },
    "key": "557057:927441f1-cc92-4030-b633-8a2bbdf7136e",
    "active": True,
    "attributes": {
        "attributes": attributes,
        "link": {
            "href": crowd_url + "/rest/usermanagement/1/user/attributes?username=existing.user&attributename=invalidPasswordAttempts&attributename=requiresPasswordChange",
            "rel": "self"
        }
    },
    "first-name": "Existing",
    "last-name": "User",
    "display-name": "Existing User",
    "email": "sample@user.cool"
}

class MockGroupCrowdUtil(object):
    def get_group_json(self, crowd_url, expanded_attributes, name='existing.group'):
        attributes=[]
        
        if expanded_attributes:
            attributes += [{
                "link": {
                    "href": crowd_url + "/rest/usermanagement/1/group/attributes?groupname="+name+"&attributename=testAttribute",
                    "rel": "self"
                },
                "name": "testAttribute",
                "values": [
                    "This is the value."
                ]
            }]
    
        return {
    "expand": "attributes",
    "link": {
        "href": "link_to_group",
        "rel": "self"
    },
    "name": name,
    "description": "Group Exists",
    "type": "GROUP",
    "active": True,
    "attributes": {
        "attributes": attributes,
        "link": {
            "href": crowd_url + "/rest/usermanagement/1/group/attributes?groupname="+name,
            "rel": "self"
        }
    }
}

    def get_child_groups_of_group_json(self, crowd_url):
        return {'groups': [
                    {'name': 'childGroup1', 
                    'link': {'rel': 'self', 
                        'href': crowd_url + '/rest/usermanagement/1/group?groupname=childGroup1'}},
                    {'name': 'childGroup2', 
                    'link': {'rel': 'self', 
                        'href': crowd_url + '/rest/usermanagement/1/group?groupname=childGroup2'}}
                    ], 'expand': 'group'}

    def get_parent_groups_of_group_json(self, crowd_url):
        return {'groups': [
                    {'name': 'parentGroup1', 
                    'link': {'rel': 'self', 
                        'href': crowd_url + '/rest/usermanagement/1/group?groupname=parentGroup1'}},
                    {'name': 'parentGroup2', 
                    'link': {'rel': 'self', 
                        'href': crowd_url + '/rest/usermanagement/1/group?groupname=parentGroup2'}}
                    ], 'expand': 'group'}

    def get_nested_members_of_group_json(self, crowd_url):
        return {
    "expand": "user",
    "users": [
        {
            "link": {
                "href": crowd_url + "/rest/usermanagement/1/user?username=sampleuser1",
                "rel": "self"
            },
            "name": "sampleuser1"
        },
        {
            "link": {
                "href": crowd_url + "/rest/usermanagement/1/user?username=sampleuser2",
                "rel": "self"
            },
            "name": "sampleuser2",
        }
    ]
}
