#!/usr/bin/python3
# 
# Copyright (c) Andrea Micheloni 2017
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import requests
from . import mock

logger = logging.getLogger(__name__)

class MockJira(mock.MockService):
    def __init__(self, location):
        mock.MockService.__init__(self, location, 'amlyYWdvZDpmb3Vyd29yZHNhbGx1cHBlcmNhc2U=')
        
    def register_uris(self, mock):
        logger.info('Registering the URIs to the mock %s', mock)
        
        self._register_POST_uri(mock, '/rest/api/2/search', request_json={'jql': 'project=UNAUTHORIZED'}, status_code = 401)
        self._register_POST_uri(mock, '/rest/api/2/search', request_json={'jql': 'project=FORBIDDEN'}, status_code = 403)
        self._register_POST_uri(mock, '/rest/api/2/search', request_json={'jql': 'project=NOTEXISTING'}, status_code = 400)

        self._register_POST_uri(mock, '/rest/api/2/search', request_json={'jql': 'project=EMPTY'}, json=MockJiraUtil().get_empty_search_result() ,status_code = 200)
        self._register_POST_uri(mock, '/rest/api/2/search', request_json={'jql': 'project=TEST'}, json=MockJiraUtil().get_search_result())

        self._register_POST_uri(mock, '/rest/api/2/issue/NOTEXISTING-1/transitions', request_json={}, status_code = 404)
        self._register_POST_uri(mock, '/rest/api/2/issue/TEST-1/transitions', request_json={'transition': {'id': 'NEX'}}, status_code = 400)
        self._register_POST_uri(mock, '/rest/api/2/issue/TEST-1/transitions', request_json={'transition': {'id': '61'}}, status_code = 204)

        self._register_POST_uri(mock, '/rest/api/2/issue/NOTEXISTING-1/comment', request_json={}, status_code = 404)
        self._register_POST_uri(mock, '/rest/api/2/issue/TEST-1/comment', request_json={}, status_code = 201)

        self._register_PUT_uri(mock, '/rest/api/2/issue/NOTEXISTING-1', request_json={}, status_code = 404)
        self._register_PUT_uri(mock, '/rest/api/2/issue/FORBIDDEN-1', request_json={}, status_code = 403)
        self._register_PUT_uri(mock, '/rest/api/2/issue/FAILED-1', request_json={}, status_code = 400)
        self._register_PUT_uri(mock, '/rest/api/2/issue/CODE-1', request_json={}, status_code = 405)
        
        self._register_PUT_uri(mock, '/rest/api/2/issue/TEST-1', request_json={'update': {'summary': [{'set': 'This is the new summary'}]}}, status_code = 204)
        
        self._register_POST_uri(mock, '/rest/jiracustomfieldeditorplugin/1/user/customfields/0001/contexts/default/options', request_json={}, status_code = 400)
        self._register_POST_uri(mock, '/rest/jiracustomfieldeditorplugin/1/user/customfields/0002/contexts/default/options', request_json={}, status_code = 500)
        self._register_POST_uri(mock, '/rest/jiracustomfieldeditorplugin/1/user/customfields/0003/contexts/default/options', request_json={}, status_code = 401)
        self._register_POST_uri(mock, '/rest/jiracustomfieldeditorplugin/1/user/customfields/0004/contexts/default/options', request_json={}, status_code = 403)
        self._register_POST_uri(mock, '/rest/jiracustomfieldeditorplugin/1/user/customfields/4711/contexts/default/options', 
                                    request_json={'optionvalue': 'new_option', 'disabled': False}, json=MockJiraUtil().get_customfield(), status_code = 200)
        
        self._register_PUT_uri(mock, '/rest/jiracustomfieldeditorplugin/1/user/customfields/0001/contexts/default/options/sort', request_json={}, status_code = 400)
        self._register_PUT_uri(mock, '/rest/jiracustomfieldeditorplugin/1/user/customfields/0002/contexts/default/options/sort', request_json={}, status_code = 500)
        self._register_PUT_uri(mock, '/rest/jiracustomfieldeditorplugin/1/user/customfields/0003/contexts/default/options/sort', request_json={}, status_code = 401)
        self._register_PUT_uri(mock, '/rest/jiracustomfieldeditorplugin/1/user/customfields/0004/contexts/default/options/sort', request_json={}, status_code = 403)
        self._register_PUT_uri(mock, '/rest/jiracustomfieldeditorplugin/1/user/customfields/4711/contexts/default/options/sort', 
                                    request_json={'order': 'ASCENDING', 'locale': 'en-US'}, json=MockJiraUtil().get_customfield(), status_code = 200)
        
        
        
class MockJiraUtil(object):
    def get_empty_search_result(self):
        return {'expand': 'schema,names', 'issues': [], 'total': 0, 'startAt': 0, 'maxResults': 50}

    def get_search_result(self):
        fields = lambda num: {'customfield_10001': None, 'progress': {'progress': 0, 'total': 0}, 'summary': 'Test issue ' + str(num), \
                               'status': {'name': 'To Do'}, 'project': {'key': 'TEST'}, 'issuetype': {'name': 'Test'}}
        issues = [{'id': '1000' + str(num), 'fields': fields(num), 'key': 'TEST-' + str(num)} for num in range(1, 4)]
        
        return {'expand': 'schema,names', 'issues': issues, 'total': 3, 'startAt': 0, 'maxResults': 50}
        
    def get_customfield(self):
        return { 'optionvalue': 'new_option', 'id': 27, 'sequence': 27, 'disabled': False }
        
